@@ -17,18 +17,31 @@
 import os
 
 from gi.module import get_introspection_module
-from gi.repository import Gdk
-from gi.repository import GLib
-from gi.repository import GObject
-from gi.repository import Pango
+from gi.repository import Gdk, GLib, GObject, Pango
 
-from meld.misc import colour_lookup_with_fallback
+from meld.style import colour_lookup_with_fallback
 from meld.treehelpers import SearchableTreeStore
 from meld.vc._vc import (  # noqa: F401
-    CONFLICT_BASE, CONFLICT_LOCAL, CONFLICT_MERGED, CONFLICT_OTHER,
-    CONFLICT_REMOTE, CONFLICT_THIS, STATE_CONFLICT, STATE_EMPTY, STATE_ERROR,
-    STATE_IGNORED, STATE_MAX, STATE_MISSING, STATE_MODIFIED, STATE_NEW,
-    STATE_NOCHANGE, STATE_NONE, STATE_NONEXIST, STATE_NORMAL, STATE_REMOVED,
+    CONFLICT_BASE,
+    CONFLICT_LOCAL,
+    CONFLICT_MERGED,
+    CONFLICT_OTHER,
+    CONFLICT_REMOTE,
+    CONFLICT_THIS,
+    STATE_CONFLICT,
+    STATE_EMPTY,
+    STATE_ERROR,
+    STATE_IGNORED,
+    STATE_MAX,
+    STATE_MISSING,
+    STATE_MODIFIED,
+    STATE_NEW,
+    STATE_NOCHANGE,
+    STATE_NONE,
+    STATE_NONEXIST,
+    STATE_NORMAL,
+    STATE_REMOVED,
+    STATE_SPINNER,
 )
 
 _GIGtk = None
@@ -90,6 +103,7 @@ class DiffTreeStore(SearchableTreeStore):
             (del_fg, roman,  bold,   True),  # STATE_REMOVED
             (del_fg, roman,  bold,   True),  # STATE_MISSING
             (unk_fg, roman,  normal, True),  # STATE_NONEXIST
+            (None,   italic, normal, None),  # STATE_SPINNER
         ]
 
         self.icon_details = [
@@ -98,7 +112,7 @@ class DiffTreeStore(SearchableTreeStore):
             ("text-x-generic", "folder", None,   None),    # NONE
             ("text-x-generic", "folder", None,   None),    # NORMAL
             ("text-x-generic", "folder", None,   None),    # NOCHANGE
-            ("dialog-warning", None,     None,   None),    # ERROR
+            ("dialog-warning-symbolic", None,     None,   None),    # ERROR
             (None,             None,     None,   None),    # EMPTY
             ("text-x-generic", "folder", new_fg, None),    # NEW
             ("text-x-generic", "folder", mod_fg, None),    # MODIFIED
@@ -107,6 +121,7 @@ class DiffTreeStore(SearchableTreeStore):
             ("text-x-generic", "folder", del_fg, None),    # REMOVED
             ("text-x-generic", "folder", unk_fg, unk_fg),  # MISSING
             ("text-x-generic", "folder", unk_fg, unk_fg),  # NONEXIST
+            ("text-x-generic", "folder", None,   None),    # SPINNER
         ]
 
         assert len(self.icon_details) == len(self.text_attributes) == STATE_MAX
@@ -121,7 +136,7 @@ class DiffTreeStore(SearchableTreeStore):
         # A folder may no longer exist, and is only tracked by VC.
         # Therefore, check the icon instead, as the pane already knows.
         icon = self.get_value(it, self.column_index(COL_ICON, pane))
-        return icon == "folder" or os.path.isdir(path)
+        return icon == "folder" or (bool(path) and os.path.isdir(path))
 
     def column_index(self, col, pane):
         return self.ntree * col + pane
@@ -177,7 +192,8 @@ class DiffTreeStore(SearchableTreeStore):
     def _find_next_prev_diff(self, start_path):
         def match_func(it):
             # TODO: It works, but matching on the first pane only is very poor
-            return self.get_state(it, 0) not in (STATE_NORMAL, STATE_NOCHANGE, STATE_EMPTY)
+            return self.get_state(it, 0) not in (
+                STATE_NORMAL, STATE_NOCHANGE, STATE_EMPTY)
 
         return self.get_previous_next_paths(start_path, match_func)
 
@@ -219,6 +235,59 @@ class DiffTreeStore(SearchableTreeStore):
             self.set(treeiter, safe_keys_values)
 
 
+class TreeviewCommon:
+
+    def on_treeview_popup_menu(self, treeview):
+        cursor_path, cursor_col = treeview.get_cursor()
+        if not cursor_path:
+            self.popup_menu.popup_at_pointer(None)
+            return True
+
+        # We always want to pop up to the right of the first column,
+        # ignoring the actual cursor column location.
+        rect = treeview.get_background_area(
+            cursor_path, treeview.get_column(0))
+
+        self.popup_menu.popup_at_rect(
+            treeview.get_bin_window(),
+            rect,
+            Gdk.Gravity.SOUTH_EAST,
+            Gdk.Gravity.NORTH_WEST,
+            None,
+        )
+        return True
+
+    def on_treeview_button_press_event(self, treeview, event):
+
+        # If we have multiple treeviews, unselect clear other tree selections
+        num_panes = getattr(self, 'num_panes', 1)
+        if num_panes > 1:
+            for t in self.treeview[:self.num_panes]:
+                if t != treeview:
+                    t.get_selection().unselect_all()
+
+        if (event.triggers_context_menu() and
+                event.type == Gdk.EventType.BUTTON_PRESS):
+
+            treeview.grab_focus()
+
+            path = treeview.get_path_at_pos(int(event.x), int(event.y))
+            if path is None:
+                return False
+
+            selection = treeview.get_selection()
+            model, rows = selection.get_selected_rows()
+
+            if path[0] not in rows:
+                selection.unselect_all()
+                selection.select_path(path[0])
+                treeview.set_cursor(path[0])
+
+            self.popup_menu.popup_at_pointer(event)
+            return True
+        return False
+
+
 def treeview_search_cb(model, column, key, it, data):
     # If the key contains a path separator, search the whole path,
     # otherwise just use the filename. If the key is all lower-case, do a
