@@ -72,6 +72,46 @@ manually_added_libs = {
     "libgdk_pixbuf-2.0-0.dll": os.path.join(sys.prefix, 'bin'),
     # librsvg is needed for SVG loading in gdkpixbuf
     "librsvg-2-2.dll": os.path.join(sys.prefix, 'bin'),
+    # Workarounds for an new not-understood cx-freeze issue
+    "libatk-1.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libbrotlicommon.dll": os.path.join(sys.prefix, "bin"),
+    "libbrotlidec.dll": os.path.join(sys.prefix, "bin"),
+    "libbz2-1.dll": os.path.join(sys.prefix, "bin"),
+    "libcairo-2.dll": os.path.join(sys.prefix, "bin"),
+    "libcairo-gobject-2.dll": os.path.join(sys.prefix, "bin"),
+    "libcrypto-3-x64.dll": os.path.join(sys.prefix, "bin"),
+    "libdatrie-1.dll": os.path.join(sys.prefix, "bin"),
+    "libepoxy-0.dll": os.path.join(sys.prefix, "bin"),
+    "libexpat-1.dll": os.path.join(sys.prefix, "bin"),
+    "libffi-8.dll": os.path.join(sys.prefix, "bin"),
+    "libfontconfig-1.dll": os.path.join(sys.prefix, "bin"),
+    "libfreetype-6.dll": os.path.join(sys.prefix, "bin"),
+    "libfribidi-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgdk-3-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgio-2.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgirepository-1.0-1.dll": os.path.join(sys.prefix, "bin"),
+    "libglib-2.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgmodule-2.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgobject-2.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libgraphite2.dll": os.path.join(sys.prefix, "bin"),
+    "libgtk-3-0.dll": os.path.join(sys.prefix, "bin"),
+    "libharfbuzz-0.dll": os.path.join(sys.prefix, "bin"),
+    "libiconv-2.dll": os.path.join(sys.prefix, "bin"),
+    "libintl-8.dll": os.path.join(sys.prefix, "bin"),
+    "liblzma-5.dll": os.path.join(sys.prefix, "bin"),
+    "libmpdec-4.dll": os.path.join(sys.prefix, "bin"),
+    "libpango-1.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpangocairo-1.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpangoft2-1.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpangowin32-1.0-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpcre2-8-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpixman-1-0.dll": os.path.join(sys.prefix, "bin"),
+    "libpng16-16.dll": os.path.join(sys.prefix, "bin"),
+    "libssl-3-x64.dll": os.path.join(sys.prefix, "bin"),
+    "libstdc++-6.dll": os.path.join(sys.prefix, "bin"),
+    "libthai-0.dll": os.path.join(sys.prefix, "bin"),
+    "libxml2-2.dll": os.path.join(sys.prefix, "bin"),
+    "zlib1.dll": os.path.join(sys.prefix, "bin"),
 }
 
 for lib, possible_path in manually_added_libs.items():
